/*****************************************************************************
** 
**	 MODULE NAME  Delaunay.C
**
**	 DESCRIPTION
**
**
**	 HISTORY
**		Name	Date	   Description
**
**		arvo	12/29/93   Copied from Dani Lischinski
**
**	 (c) Copyright 1993
**		 Program of Computer Graphics, Cornell University, Ithaca, NY
**		 ALL RIGHTS RESERVED
**
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "../util/Vec2.h"
#include "../util/Delaunay.h"
#include "../util/quadedge.h"

#undef X
#undef Y
#undef Z
static const float DefaultSkipTol = 0.005;


static Mesh *mesh_from_box( const Box2d &box )
{
	float xmin = box.Xmin();
	float xmax = box.Xmax();
	float ymin = box.Ymin();
	float ymax = box.Ymax();
	
	Mesh *m = new Mesh( 
		Point2d( xmin, ymin ), 
		Point2d( xmax, ymin ),
		Point2d( xmax, ymax ),
		Point2d( xmin, ymax )
		);
	
	// Add three more points within each of the four edges.
	
	for( int i = 1; i <= 3; i++ )
	{
		float x = xmin + i * ( xmax - xmin ) / 4.0;
		float y = ymin + i * ( ymax - ymin ) / 4.0;
		m->InsertBoundaryPoint( Point2d( x, ymin ) );
		m->InsertBoundaryPoint( Point2d( x, ymax ) );
		m->InsertBoundaryPoint( Point2d( xmin, y ) );
		m->InsertBoundaryPoint( Point2d( xmax, y ) );
	}
	
	
	return m;
}

DelaunayMesh::DelaunayMesh( const Box2d &box_ )
{
	SkipTol = DefaultSkipTol;
	box = box_;
	mesh = mesh_from_box( box );
}

DelaunayMesh::DelaunayMesh()
{
	SkipTol = DefaultSkipTol;
	Vec2 a( 0.0, 0.0 );
	Vec2 b( 1.0, 1.0 );
	box.Set( a, b );
	mesh = mesh_from_box( box );
}

DelaunayMesh::~DelaunayMesh()
{
	Mesh *m = (Mesh *)mesh;
	delete m;
}

void DelaunayMesh::Add( const Multiline &mline, float value )
{
	Mesh *m = (Mesh *)mesh;
	Pnode *p = mline.First();
	for( Pnode *q = p->Next(); q != NULL; q = q->Next() )
	{
		if (! q->Jump() )
			m->InsertEdge( 
			Point2d( p->X(), p->Y() ),
			Point2d( q->X(), q->Y() ),
			value, value );
		p = q;
	}
}

void DelaunayMesh::Add( const Vec2 &p, const Vec2 &q, float value1, float value2 )
{
	Mesh *m = (Mesh *)mesh;
	m->InsertEdge( 
		Point2d( p.X(), p.Y() ), 
		Point2d( q.X(), q.Y() ), 
		value1, value2 );
}

void DelaunayMesh::Add( const Vec2 &p, float value )
{
	Mesh *m = (Mesh *)mesh;
	m->InsertValue( Point2d( p.X(), p.Y() ), value );
}

void DelaunayMesh::Draw( DrawLine_t func )
{
	Mesh *m = (Mesh *)mesh;
	if( m != NULL ) m->DrawLines( func );
}

void DelaunayMesh::Shade( DrawTri_t func )
{
	Mesh *m = (Mesh *)mesh;
	if( m != NULL ) m->DrawTriangles( func );
}

void DelaunayMesh::TraverseTris( TraverseTri_t func, void *data )
{
	Mesh *m = (Mesh *)mesh;
	if( m != NULL ) m->TraverseTris( func, data );
}

void DelaunayMesh::TraverseEdges( TraverseEdge_t func, void *data )
{
	Mesh *m = (Mesh *)mesh;
	if( m != NULL ) m->TraverseEdges( func, data );
}

void DelaunayMesh::Clear()
{
	Mesh *m = (Mesh *)mesh;
	delete m;
	mesh = mesh_from_box( box );
}

void DelaunayMesh::Clear( const Box2d &b )
{
	Mesh *m = (Mesh *)mesh;
	delete m;
	box = b;
	mesh = mesh_from_box( box );
}

#include "stdafx.h"
