// DirectoryTree.cpp: implementation of the DirectoryTree class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"

#include "DirectoryTree.h"
#include <string.h>
#include <windows.h>
#include <iostream>
using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

DT_File::DT_File(char *file, int filesize, DT_Directory *dirloc)
{
	if (file == NULL)
		name = NULL;
	else
	{
		name = new char[strlen(file)+1];
		strcpy(name, file);
	}
	next = NULL;
	size = filesize;
	dir = dirloc;
}

char *DT_File::GetFullName()
{
	char *tmp, *ret;
	tmp = dir->GetFullName();
	ret = new char[strlen(tmp) + strlen(name) + 2];
	strcpy(ret, tmp);
	strcat(ret, "\\");
	strcat(ret, name);
	return ret;
}

bool DT_File::HasSuffix(char *suffix)
{
	size_t l = strlen(name), s = strlen(suffix);
	if (l < s) return false;
	return (strncmp(name + l - s, suffix, s) == 0);
}

DT_Directory::DT_Directory(char *dir, DT_Directory *parentdir, DirectoryTree *roottree)
{
	if (dir == NULL)
		name = NULL;
	else
	{
		name = new char[strlen(dir)+1];
		strcpy(name, dir);
	}
	parent = parentdir;
	child = NULL;
	next = NULL;
	root = roottree;
	files = NULL;
}

char *DT_Directory::GetRelName()
{
	if (parent == NULL) return "";

	size_t s;
	char *tmp, *ret;

	tmp = parent->GetRelName();
	s = strlen(tmp);
	ret = new char[s + strlen(name) + 2];
	strcpy(ret, tmp);
	if (s != 0)
		strcat(ret, "\\");
	strcat(ret, name);
	return ret;
}

char *DT_Directory::GetFullName()
{
	size_t s, t;
	char *tmp, *ret;
	tmp = GetRelName();
	s = strlen(tmp);
	t = strlen(root->dirname);
	ret = new char[s + t + 1];
	strcpy(ret, root->dirname);
	strcat(ret, tmp);
	if (s == 0)
		ret[t-1] = 0;
	return ret;
}


bool DT_Directory::Populate(bool recursive)
{
	BOOL more, isdir;
	int size;
	char *path, *name;
	DT_Directory *curdir;
	DT_File *curfile;
	HANDLE hFind;
	WIN32_FIND_DATA finddata;

	name = GetFullName();
	path = new char[strlen(name)+5];
	strcpy(path, name);
	strcat(path, "\\");
	strcat(path, "*.*");

	// now scan the directory
	hFind = FindFirstFile(path, &finddata);
	more = (hFind != (HANDLE)-1);
	while (more)
	{
		isdir = (finddata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
		name = finddata.cFileName;
		size = finddata.nFileSizeLow;
		// if we're adding a file
		if (!isdir)
		{
			// add the first file
			if (files == NULL)
			{
				files = new DT_File(name, size, this);
				curfile = files;
			}
			// add the next file
			else
			{
				curfile->next = new DT_File(name, size, this);
				curfile = curfile->next;
			}
		}
		// otherwise, add a directory (but not . or ..)
		else if (strcmp(name, ".") != 0 && strcmp(name, "..") != 0)
		{
			// add the first child
			if (child == NULL)
			{
				child = new DT_Directory(name, this, root);
				curdir = child;
				if (recursive)
					curdir->Populate();
			}
			// add the next child
			else
			{
				curdir->next = new DT_Directory(name, this, root);
				curdir = curdir->next;
				if (recursive)
					curdir->Populate();
			}
		}
		more = FindNextFile(hFind, &finddata);
	}

	return true;
}

void DT_Directory::PrintFiles()
{
	DT_File *file = files;
	
	while (file != NULL)
	{
		cout << file->name << "\t" << file->size << endl;
		file = file->next;
	}
}

void DT_Directory::PrintSubdirs()
{
	DT_Directory *dir = child;
	
	while (dir != NULL)
	{
		cout << dir->name << endl;
		dir = dir->next;
	}
}

void DT_Directory::PrintAll(bool recursive)
{
	cout << "Subdirectories in " << GetFullName() << ":" << endl;
	PrintSubdirs();
	cout << "Files in " << GetFullName() << ":" << endl;
	PrintFiles();
	cout << endl;
	if (!recursive) return;

	DT_Directory *dir = child;
	while (dir != NULL)
	{
		dir->PrintAll(recursive);
		dir = dir->next;
	}
}

DirectoryTree::DirectoryTree(char *dir)
{
	root = NULL;
	if (dir == NULL) dirname = NULL;
	else ScanDirectory(dir);
	changedir = false;
}

DirectoryTree::~DirectoryTree()
{
	if (root != NULL)
		delete root;
}

bool DirectoryTree::IsValidFile(const char *path)
{
	DWORD dwAttrib = GetFileAttributes(path);
	if (dwAttrib == -1)
	{
		return false;
	}
	if (dwAttrib & FILE_ATTRIBUTE_DIRECTORY)
	{
		return false;
	}
	return true;
}

bool DirectoryTree::IsValidDir(const char *path)
{
	DWORD dwAttrib = GetFileAttributes(path);
	if (dwAttrib == -1)
	{
		return false;
	}
	if (dwAttrib & FILE_ATTRIBUTE_DIRECTORY)
	{
		return true;
	}
	return false;
}

void DirectoryTree::ConvertBackslashes(char *path)
{
	size_t i, j = strlen(path);
	for (i=0; i<j; i++)
	{
		if (path[i] == '\\')
			path[i] = '/';
	}
}

bool DirectoryTree::ScanDirectory(const char *dir)
{
	if (dir == NULL) return false;

	size_t size = strlen(dir);
	dirname = new char[size+2];
	strcpy(dirname, dir);
	if (dirname[size-1] != '\\' && dirname[size-1] != '/')
		strcat(dirname, "\\");

	root = new DT_Directory("", NULL, this);
	return root->Populate();
}

char *DirectoryTree::GetRoot()
{
	char *ret = new char[strlen(dirname)+1];
	strcpy(ret, dirname);
	return ret;
}

DT_File *DirectoryTree::GetFirstFile()
{
	if (root->files != NULL)
	{
		curfile = root->files;
		return curfile;
	}
	curfile = NULL;
	return GetNextFile();
}

DT_File *DirectoryTree::GetNextFile()
{
	DT_Directory *dir;
	changedir = false;

	if (curfile == NULL)
	{
		dir = root;
	}
	else
	{
		// if there's another file in the same directory
		if (curfile != NULL && curfile->next != NULL)
		{
			curfile = curfile->next;
			return curfile;
		}
		dir = curfile->dir;
		changedir = true;
	}

	// otherwise we have to scan subdirectories
	do
	{
		if (dir->child == NULL)
		{
			if (dir->next != NULL)
			{
				dir = dir->next;
				continue;
			}
			while (dir->next == NULL)
			{
				dir = dir->parent;
				// if there are no more files in the list
				if (dir == NULL)
				{
					curfile = NULL;
					return curfile;
				}
			}
			dir = dir->next;
		}
		else
		{
			dir = dir->child;
		}
	} while (dir->files == NULL);
	curfile = dir->files;
	return curfile;
}

bool DirectoryTree::ChangedDir()
{
	return changedir;
}

void DirectoryTree::PrintTree()
{
	root->PrintAll();
}
