// DirectoryTree.h: interface for the DirectoryTree class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _DIRECTORYTREE_H
#define _DIRECTORYTREE_H

#include <stdlib.h>

class DirectoryTree;
class DT_File;
class DT_Directory;


// File class - stores a file and is part of a linked
// list of other files in the same directory
class DT_File
{
public:
	DT_File *next;     // the next file in the linked list
	char *name;        // name of this file
	int size;          // file size
	DT_Directory *dir; // the directory this file is in

public:
	DT_File(char *file = NULL, int filesize = 0, DT_Directory *dirloc = NULL);
//	~DT_File();

	// returns the full path name of this file
	char *GetFullName();

	// returns true if the file name has the given suffix
	bool HasSuffix(char *suffix);
};


// Directory class - stores a directory node in a tree
// and is part of a linked list of its siblings
class DT_Directory
{
public:
	// Directory tree pointers
	DT_Directory *parent; // the parent directory
	DT_Directory *child;  // linked list of subdirectories
	DT_Directory *next;   // the next dir in this linked list
	DirectoryTree *root;  // access to the root of the tree

	// Directory information
	char *name;           // the directory name
	DT_File *files;       // linked list of files in this dir

public:
	DT_Directory(char *dir = NULL, DT_Directory *parentdir = NULL, DirectoryTree *roottree = NULL);
//	~DT_Directory();

	// returns the name of this directory relative to the root
	char *GetRelName();

	// returns the full path name of this directory
	char *GetFullName();

	// populates the directory with entries from the file system
	bool Populate(bool recursive = true);

	// print the contents of the directory, recursively if desired
	void PrintAll(bool recursive = true);

	// print a simple list of all files in this directory
	void PrintFiles();

	// print a list of all the subdirectories of this directory
	void PrintSubdirs();
};


// DirectoryTree class - is the root of a directory tree
// structure that stores all the files and subdirectories
// under a given directory name
class DirectoryTree
{
public:
	DirectoryTree(char *dir = NULL);
	~DirectoryTree();

	// these return true if the given string is a valid file or directory
	static bool IsValidFile(const char *path);
	static bool IsValidDir(const char *path);

	// converts all backslashes in a path name to forward slashes
	static void ConvertBackslashes(char *path);

	// scans a given directory and all it subdirectories and stores
	// them in a directory tree
	bool ScanDirectory(const char *dir);

	// returns the name of the root directory
	char *GetRoot();

	// returns the first file in the list
	DT_File *GetFirstFile();

	// returns the next file in the list
	DT_File *GetNextFile();

	// returns true if the last call to GetNextFile() had to search
	// a new subdirectory to find the file
	bool ChangedDir();

	// prints the list of files in the scanned directory
	void PrintTree();

protected:
	bool changedir;
	char *dirname;
	DT_Directory *root;
	DT_File *curfile;

	friend class DT_File;
	friend class DT_Directory;
};

#endif
