/***************************************************************************
* Math.h                                                                   *
*                                                                          *
* Convenient constants, macros, and inline functions for basic math        *
* functions.                                                               *
*                                                                          *
*   HISTORY                                                                *
*      Name    Date        Description                                     *
*                                                                          *
*      arvo    06/17/93    Initial coding.                                 *
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#ifndef __MATH_INCLUDED__
#define __MATH_INCLUDED__

#include <math.h>
#include <stdlib.h>

#ifndef MAXFLOAT
#define MAXFLOAT 1.0E+20
#endif

static const double
    Pi            = 3.14159265358979f,
    PiSquared     = Pi * Pi,
    TwoPi         = 2.0f * Pi,
    FourPi        = 4.0f * Pi,
    PiOverTwo     = Pi / 2.0f,
    PiOverFour    = Pi / 4.0f,
    OverPi        = 1.0f / Pi,
    OverTwoPi     = 1.0f / TwoPi,
    OverFourPi    = 1.0f / FourPi,
    Infinity      = MAXFLOAT,
    Tiny          = 1.0f / MAXFLOAT,
    DegreesToRad  = Pi / 180.0f,
    RadToDegrees  = 180.0f / Pi;

inline int     Odd   ( int    k           ) { return k & 1; }
inline int     Even  ( int    k           ) { return !(k & 1); }
inline double  Abs   ( int    x           ) { return x > 0  ? double(x) : double(-x); }
inline double  Abs   ( double x           ) { return x > 0. ? x : -x; }
inline int     Min   ( int    x, int    y ) { return x < y ? x : y; }
inline double  Min   ( double x, double y ) { return x < y ? x : y; }
inline int     Max   ( int    x, int    y ) { return x > y ? x : y; }
inline double  Max   ( double x, double y ) { return x > y ? x : y; }
inline double  dMin  ( double x, double y ) { return x < y ? x : y; }
inline double  dMax  ( double x, double y ) { return x > y ? x : y; }
inline double  Sqr   ( int    x           ) { return double(x * x); }
inline double  Sqr   ( double x           ) { return x * x; }
inline double  Sqrt  ( double x           ) { return x > 0. ? sqrt(x) : 0.0f; }
inline double  Cubed ( double x           ) { return x * x * x; }
inline int     Sign  ( double x           ) { return x > 0. ? 1 : (x < 0. ? -1 : 0); }
inline void    Swap  ( double &a, double &b ) { double c = a; a = b; b = c; }
inline void    Swap  ( int    &a, int    &b ) { int   c = a; a = b; b = c; }
inline double  Sin   ( double x, int     n ) { return pow( sin(x), n ); }
inline double  Cos   ( double x, int     n ) { return pow( cos(x), n ); }
inline double  ToSin ( double x            ) { return Sqrt( 1.0f - Sqr(x) ); }
inline double  ToCos ( double x            ) { return Sqrt( 1.0f - Sqr(x) ); }
inline double  MaxAbs( double x, double y  ) { return Max( Abs(x), Abs(y) ); }
inline double  MinAbs( double x, double y  ) { return Min( Abs(x), Abs(y) ); }
inline double  Pythag( double x, double y  ) { return Sqrt( x*x + y*y ); }

inline double ArcCos( double x )
    {
    double y;
    if( -1.0 <= x && x <= 1.0 ) y = acos( x );
    else if( x >  1.0 ) y = 0.0;
    else if( x < -1.0 ) y = Pi;
    return y;
    }

inline double ArcSin( double x )
    {
    if( x < -1.0 ) x = -1.0;
    if( x >  1.0 ) x =  1.0;
    return asin( x );
    }

inline double Clamp( double min, double &x, double max )
    {
    if( x < min ) x = min; else
    if( x > max ) x = max;
    return x;
    }

inline double Max( double x, double y, double z )
    {
    double t;
    if( x >= y && x >= z ) t = x;
    else if( y >= z ) t = y;
    else t = z;
    return t;
    }

inline double Min( double x, double y, double z )
    {
    double t;
    if( x <= y && x <= z ) t = x;
    else if( y <= z ) t = y;
    else t = z;
    return t;
    }

inline double dMax( double x, double y, double z )
    {
    double t;
    if( x >= y && x >= z ) t = x;
    else if( y >= z ) t = y;
    else t = z;
    return t;
    }

inline double dMin( double x, double y, double z )
    {
    double t;
    if( x <= y && x <= z ) t = x;
    else if( y <= z ) t = y;
    else t = z;
    return t;
    }

inline double MaxAbs( double x, double y, double z )
    {
    return Max( Abs( x ), Abs( y ), Abs( z ) );
    }

inline double Pythag( double x, double y, double z )
    {
    return sqrt( x * x  +  y * y  +  z * z );
    }

extern double  ArcTan          ( double x, double y      );
extern double  ArcQuad         ( double x, double y      );
extern double  MachineEpsilon  (                       );
extern double  LogGamma        ( double x               );
extern double  LogFact         ( int n                 );
extern double  LogDoubleFact   ( int n                 );   // log( n!! )
extern double  BinomialCoeff   ( int n, int k          );
extern void   BinomialCoeffs  ( int n, long   *coeffs );
extern void   BinomialCoeffs  ( int n, double *coeffs  );
extern double  TrinomialCoeff  ( int i, int j, int k   );
extern double  MultinomialCoeff( int k, int N[]        );
extern double  RelErr          ( double x, double y      );

#ifndef ABS
#define ABS( x ) ((x) > 0 ? (x) : -(x))
#endif

#ifndef MAX
#define MAX( x, y ) ((x) > (y) ? (x) : (y))
#endif

#ifndef MIN
#define MIN( x, y ) ((x) < (y) ? (x) : (y))
#endif

#endif







